(define-module (ordo action remote-cmd)
  #:use-module (ice-9 exceptions)
  #:use-module (ice-9 optargs)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-71)
  #:use-module (ordo connection)
  #:use-module (ordo connection base)
  #:use-module (ordo logger)
  #:use-module (ordo util flatten)
  #:export (remote-cmd))

(define (remote-cmd conn prog . args)
  (let ((args options (break keyword? args)))
    (let-keywords
     options #t
     ((return identity)
      (check? #f))
     (let ((command (build-command conn prog (remove unspecified? (flatten args)) options)))
       (log-msg 'DEBUG "Running command: " command " on connection " (describe conn))
       (let ((out rc (remote-exec conn command)))
         (log-msg 'DEBUG "Command exit code: " rc)
         (if check?
             (if (zero? rc)
                 (return out)
                 (raise-exception (make-exception
                                   (make-external-error)
                                   (make-exception-with-message (format #f "Non-zero exit (~a) from ~a" rc prog))
                                   (make-exception-with-irritants out))))
             (values (return out) rc)))))))
