#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo connection base)
  #:use-module (ice-9 match)
  #:use-module (ice-9 optargs)
  #:use-module (oop goops)
  #:use-module (ordo util flatten)
  #:use-module (ordo util shell-quote)
  #:use-module ((srfi srfi-1) #:select (remove))
  #:export (<connection>
            describe
            setup
            teardown
            build-command
            remote-exec
            with-remote-input-file
            with-remote-output-file))

(define-generic setup)
(define-generic teardown)
(define-generic describe)
(define-generic build-command)
(define-generic remote-exec)
(define-generic with-remote-input-file)
(define-generic with-remote-output-file)

(define-class <connection> ())

(define-method (setup (c <connection>)) #t)

(define-method (teardown (c <connection>)) #t)

(define-method (build-command (c <connection>) (prog-name <string>) (prog-args <list>) (options <list>))
  (let-keywords
   options #t
   ((pwd #f)
    (env #f)
    (shell-quote? #t)
    (redirect-err? #t))
   (let ((xs (remove unspecified?
                     (flatten (list "env"
                                    (when pwd (list "--chdir" (string-shell-quote pwd)))
                                    (when env (map (match-lambda ((k . v) (string-append k "=" (string-shell-quote v)))) env))
                                    prog-name
                                    (if shell-quote? (map string-shell-quote prog-args) prog-args)
                                    (when redirect-err? "2>&1"))))))
     (string-join xs " "))))
